/********************************************************/
/*                  prj.interface.c                     */
/********************************************************/
/*     L'Architecte et le Minotaure (Un aperçu 3D)      */
/********************************************************/

/* Inclusion des librairies standard */
#include<stdio.h>
#include<stdlib.h>
#include <math.h>

/* Pour vérifier si le fichier a été modifié */
#include <sys/stat.h>

/* Inclusion des fichiers d'en-tête glut */
#include <GL/glut.h>

/* Déclaration des fonctions de rappel */
void affichage();
void clavier(unsigned char key,int x, int y);
void sourisClick(int bouton,int state,int x,int y);
void sourisBouge(int x,int y);
void enAttente ();

/* Déclaration des fonctions utiles */
void calcCosSinTable();
void changeZoom();

/* Déclaration des fonctions d'importation */
void importFichier ();
void lireLigne (FILE* f, char* chaine);

/* Le type cube */
typedef struct{
  int x; // Abcisse
  int y; // Ordonnée
  int z; // Hauteur
  float r; // Composante rouge
  float v; // Composante verte
  float b; // Composante bleue
}cube;

/* Les variables globales */
float py=0.0, px=0.0, tabsin[360], tabcos[360], theta=50;
int xold, r=0, nbcubes=0, dernmodif=0;
cube* tabcubes;

int main(int argc,char **argv){

  /*Infos pour l'utilisateur */
  printf ("Avancer : [z]\n");
  printf ("Reculer : [s]\n");
  printf ("Pas à gauche : [q]\n");
  printf ("Pas à droite : [d]\n");
  printf ("Tourner : [ClickG+Mvt]\n");
  printf ("Zoom avant : [n]\n");
  printf ("Zoom arrière : [b]\n");
  printf ("Quitter : [o]\n");

  /* Initialisation de glut et creation
     de la fenetre */
  glutInit(&argc,argv);
  // Avec 1 tampon de cache et 1 tampon de profondeur
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
  glutInitWindowPosition(0,0);
  glutInitWindowSize(400,400);
  glutCreateWindow("L'Architecte et le Minotaure (Affichage 3D)");

  /* Initialisation d'OpenGL */
  glClearColor(0.0,0.0,0.0,0.0);
  changeZoom();
  glPointSize (2.0);
  glEnable (GL_DEPTH_TEST); // Activation du tampon de profondeur

  /* Precalcul des sinus et cosinus */
  calcCosSinTable();

  /* enregistrement des fonctions de rappel */
  glutDisplayFunc(affichage);
  glutKeyboardFunc(clavier);
  glutMouseFunc(sourisClick);
  glutMotionFunc(sourisBouge);
  glutIdleFunc(enAttente);

  /* Entre dans la boucle principale glut */
  glutMainLoop();
  return 0;
}

void affichage(){

  int i;

  /* Effacement de l'image avec la couleur de fond */
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();

  /* On veut le plan (x,y) horizontal et l'axe z vertical */
  glRotated (-90, 1, 0, 0);

  /* On se place légèrement en hauteur */
  glTranslated (0, 0, -1);
  
  /* Application des transfos de visualisation */
  glRotated (r, 0, 0, 1);
  glTranslatef (-px, -py, 0);
  
  /* Empilage de la matrice des transformations
     de visualisation, elle reservira pour chaque
     objet */
  glPushMatrix();

  /* Création des objets */
  
  /* Création d'un plan horizontal pour le chantier de construction */
  glTranslatef (0, 0, -0.5);
  glScalef (1.0, 1.0, 0.05);
  glColor3d (1, 1, 1);
  glutSolidCube (20);

  glPopMatrix();
  glPushMatrix();

  /* Création d'un plan horizontal pour la zone de stockage */
  glTranslatef (0, 20, -0.5);
  glScalef (1.0, 1.0, 0.05);
  glColor3f (0.8, 0.8, 0.8);
  glutSolidCube (20);

  glPopMatrix();
  glPushMatrix();

  /* Création d'un plan horizontal pour la zone de réparation */
  glTranslatef (0, -20, -0.5);
  glScalef (1.0, 1.0, 0.05);
  glColor3f (0.6, 0.6, 0.6);
  glutSolidCube (20);

  /* Importation des données */
  importFichier ();
  
  /* Création des cubes */
  for (i=0; i<nbcubes; i++){
    /* On dépile la matrice pour obtenir uniquement
       les trasformations de visualisation, puis on
       la rempile */
    glPopMatrix ();
    glPushMatrix ();
    glTranslated (tabcubes[i].x, tabcubes[i].y, tabcubes[i].z);
    glColor3f (tabcubes[i].r, tabcubes[i].v, tabcubes[i].b);
    glutSolidCube (2);
  }

  /* On libère la pile */
  glPopMatrix ();

  /* On force l'affichage du resultat */
  glFlush();
  glutSwapBuffers();
}


void clavier(unsigned char key,int x, int y) {

  switch (key){

  case 'o': // Sortie du programme
    exit (0);
    
  case 'z': // L'utilisateur avance
    py += 0.2*tabcos[r];
    px += 0.2*tabsin[r];
    glutPostRedisplay();
    break;
    
  case 's': // L'utilisateur recule
    py -= 0.2*tabcos[r];
    px -= 0.2*tabsin[r];
    glutPostRedisplay();
      break;
      
  case 'q': // L'utilisateur fait un pas sur la gauche
    py += 0.2*tabsin[r];
    px -= 0.2*tabcos[r];
    glutPostRedisplay();
    break;
    
  case 'd': // L'utilisateur fait un pas sur la droite
    py -= 0.2*tabsin[r];
    px += 0.2*tabcos[r];
    glutPostRedisplay();
    break;
    
  case 'b':
    theta+=1;
    if (theta>180)
      theta=180;
    changeZoom();
    glutPostRedisplay();
    break;
    
  case 'n':
    theta-=1;
    if (theta<0)
      theta=0;
    changeZoom();
    glutPostRedisplay();
    break;
    
  }
}

void sourisClick(int bouton,int state,int x,int y){

  if (state==GLUT_DOWN){
     xold=x;
  }
}

void sourisBouge (int x,int y){

  r+=x-xold;
  if (r>=360)
    r-=360;
  if (r<0)
    r=360+r;
  xold=x;
  glutPostRedisplay();

}

void calcCosSinTable (){

  int i;
  for (i=0;i<360;i++){
    tabsin[i]=sin(i/360.0*6.283185);
    tabcos[i]=cos(i/360.0*6.283185);
  }
}

void changeZoom(){

  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();
  gluPerspective (theta, 1.0 ,0.1, 40.0);
  glMatrixMode (GL_MODELVIEW);

}

void importFichier (){

  static char* nomfichier = "prj.interface.txt";
  struct stat statb;

  if (stat(nomfichier, &statb) == 0){
    
    if (statb.st_mtime != dernmodif){

      dernmodif = statb.st_mtime;

      FILE* fichier;
      char uneligne[255];
      int i, x ,y, z;
      float r, v, b;

      // Ouverture du fichier d'interface
      fichier = fopen ("prj.interface.txt", "rt");
      
      // Lecture du nombre de cubes
      lireLigne (fichier, uneligne);
      sscanf (uneligne, "(nbcubes= %d)\n", &nbcubes);

      tabcubes = (cube*) malloc (nbcubes * sizeof (cube));
      
      // Lecture de tous les cubes
      for (i=0; i<nbcubes; i++){
	lireLigne (fichier, uneligne);
	sscanf (uneligne, "(x= %d y= %d z= %d r= %f v= %f b= %f)\n", &x, &y, &z, &r, &v, &b);
	tabcubes[i].x = x;
	tabcubes[i].y = y;
	tabcubes[i].z = z;
	tabcubes[i].r = r;
	tabcubes[i].v = v;
	tabcubes[i].b = b;
      }
      
      // Fermeture du fichier d'interface
      fclose (fichier);    
    }
  }

  else
    nbcubes = 0;

}

void lireLigne (FILE* f, char* chaine){

  do{
    fgets (chaine, 255, f);
  }while (chaine[0] == '\n');

}

void enAttente (){

  glutPostRedisplay();

}
